
// Special groups (implemented by the seeds, specified as a note here) :
// 'root' group, ancestor of all SimpleGroups. (simple group)
// 'noone' group, direct child of 'root' group. Serves as recipient of all unpublished Messages 
//      (e.g. drafts, messages with deleted recipient...) (simple group)

let schoolEnum = [
    'polytechnique',
    'ensta',
    'supoptique',
    'ensae',
    'centrale',
    'enscachan',
    'otherschool',
    'notaschool'
];

exports.up = function (knex, Promise) {
    return knex.schema.createTable('groups', function (table) {
        table.string('gid',128).primary().notNullable(); //primary key *human readable*! should, but does not need to, match exactly column 'name'
        table.timestamps(true, true); //adds timestamptz-type (a PostgresQL data type) updated_at and created_at columns
        
        table.enum('type', ['simple', 'meta', 'error']).notNullable().defaultTo('error');
        table.string('parent_gid',128).notNullable().defaultTo('root')
            .references('gid').inTable('groups') 
            .onDelete('SET DEFAULT'); //if parent group is deleted, become child of root group
        
        table.string('name').notNullable();
        table.string('website'); //.defaultTo(''); <-- default to null instead
        table.text('description').defaultTo('Hello world! <Insert group description here>');

        table.enum('school', schoolEnum).notNullable();
    });
};

exports.down = function (knex, Promise) {
    return knex.schema.dropTable('groups');
};
