// Using associative tables is the recommended way of representing many-to-many relationships.
//https://en.wikipedia.org/wiki/Many-to-many_%28data_model%29
//http://web.csulb.edu/colleges/coe/cecs/dbdesign/dbdesign.php?page=manymany.php

let rightsEnum = [
    'admin',
    'speaker',
    'member'
];

exports.up = function (knex, Promise) {
    return knex.schema.createTable('metagroup_memberships', function (table) {
        table.timestamps(true, true);

        table.string('simple_group_gid',128).notNullable()
            .references('gid').inTable('groups')
            .onDelete('CASCADE'); //delete row from group_memberships if member is deleted
        table.string('meta_group_gid',128).notNullable()
            .references('gid').inTable('groups')
            .onDelete('CASCADE');
        // It is up to the user to check that the inserted gids do correspond 
        //      to a simple_group and a meta_group (respectively).
        // There is absolutely no check for this on the database side.
        
        table.enum('rights', rightsEnum).notNullable(); //TODO: this is not used by GraphQL schema!
    });
};

exports.down = function (knex, Promise) {
    return knex.schema.dropTable('metagroup_memberships');
};
