
exports.up = function(knex, Promise) {
    return knex.schema.createTable('messages_answers', function (table) {
        table.increments('mid'); //autoincrementing (non-nullable) primary key
        table.timestamps(true, true); //adds timestamptz-type (a PostgresQL data type) updated_at and created_at columns

        table.string('title').notNullable();
        table.text('content');
        
        table.string('recipient',128).notNullable().defaultTo('noone')
            .references('gid').inTable('groups')
            .onDelete('SET DEFAULT'); //if recipient is deleted, direct to the special "no-one" group
        /*
        we require that a GraphQL Answer's author field be the same as its recipient field.
        since recipient is already known (defined in the 'messages' parent-table),
        then we simply don't store answers' authors in database.
        (another way would be to add equality constraints, but uselessly complex.)
        table.string('author', 128).notNullable()
            .references('gid').inTable('groups')
            .onDelete('CASCADE');
        */

        table.integer('for_question').notNullable()
            .references('mid').inTable('messages_questions')
            .onDelete('CASCADE'); //if question is deleted, also delete the answer
    
    }).then(() => {
        //update 'questions' table by adding the 'for_answer' column
        knex.schema.table('questions', function (table) {
            table.integer('for_answer')
                .references('mid').inTable('messages_answers')
                .onDelete('SET NULL'); //if answer is deleted, set for_answer to null
        });
    });
};

exports.down = function(knex, Promise) {
    return knex.schema.dropTable('messages_answers');
};
